function [mybetainv,myfsigzero,mytheta,logvzeromat,medsig2vals,gdist,fgonbin,riskavbin,databig,covmat]=Structural_Estimation_with_Covariates(id,sigma2,risktolerance,weights,covmat,sigbinrange)

%% Define Variables
scalewts=length(weights)*(weights./(sum(weights)));  % normalizes weights to sum to total number of observations
assignin('base', 'scalewts', scalewts)

%% Risk Aversion Measurement Error Parameters from KSS Paper
mu=-1.05;
sig2x=.76;
sig2eps=1.69;

%% Here I generate the unconditional (weighted) distribution of sigma^2 into the 10 bins

% s2bins holds the fraction of weighted observations in each sigma2 bin  
[s2bins,medsig2vals_new,counted]=s2tenbins(sigbinrange,sigma2,scalewts);
assignin('base', 's2bins', s2bins)

%% Setup Cont. - Define and Populate Risk Aversion Bins

%risktolval=[.15,.28,.35,.57];
risktolval=[.57,.35,.28,.15];

%riskavbin4 is most risk averse, riskavbin1 is least risk averse
riskavbin=ones(size(sigma2));
parfor i=1:length(riskavbin)
    if risktolerance(i)<.1501 && risktolerance(i)>.1499
        riskavbin(i)=4;
    elseif risktolerance(i)<.2801 && risktolerance(i)>.2799
        riskavbin(i)=3;
    elseif risktolerance(i)<.3501 && risktolerance(i)>.3499
        riskavbin(i)=2;
    end
end

%% Set up risk tolerance values and generate gamma/sigma pdfs

% gsPDF is the weighted (# of obs) 10x4 joint distribution of sigma2 and gamma by bins (note that risktol is translated to gamma here)
% Note: gsPDF sums to number of observations, so everyone is counted
[gsPDF,counted1]=structuralGSpdf(id,sigma2,risktolerance,scalewts,sigbinrange,risktolval);
%counted1

% papergsPDF is 7 column matrix: first two columns are upper/lower bounds of sigma2 bins, third column is unconditional distribution by RA bin, 
% last 4 columns are distribution of sigma2 by risk aversion bin (each bin sums to 1). Bin 1 (column 3) is least risk averse.
papergsPDF=[gsPDF(:,1)/sum(gsPDF(:,1)),gsPDF(:,2)/sum(gsPDF(:,2)),gsPDF(:,3)/sum(gsPDF(:,3)),gsPDF(:,4)/sum(gsPDF(:,4))];
papergsPDF=[sqrt(sigbinrange(:,1:2)),s2bins,papergsPDF];

assignin('base', 'papergsPDF', papergsPDF)

%% Generate sig2/risk_aversion bins, means and st. deviations of sigma2 by risk aversion bin

% Reported in Table 3: sig2rabin1 is (weighted) sig2 values for all risk aversion 1 bin folks, same for sig2rabin2, etc.
[sig2rabin1,sig2rabin2,sig2rabin3,sig2rabin4,numobsbin1,numobsbin2,numobsbin3,numobsbin4]...
    =gen_sig_mean_std(sigma2,riskavbin,scalewts);

% print Table 2 data to screen
Table_2=[papergsPDF;[0 0 sum(scalewts) round(numobsbin1) round(numobsbin2) round(numobsbin3) round(numobsbin4)];...
    [0 0 counted1/sum(scalewts) numobsbin1/sum(scalewts) numobsbin2/sum(scalewts) numobsbin3/sum(scalewts) numobsbin4/sum(scalewts)]];
display('Table 2')
display('----------------------------------------------------------------------------')
fprintf('%10.4f %10.4f %10.4f %10.4f %10.4f %10.4f %10.4f\n',Table_2(:,1:7)')

% output sig2rabins to working memory
assignin('base', 'sig2rabin1', sig2rabin1)
assignin('base', 'sig2rabin2', sig2rabin2)
assignin('base', 'sig2rabin3', sig2rabin3)
assignin('base', 'sig2rabin4', sig2rabin4)

%% 10xN "big data" for structural estimation

% databig is a 10x(# of obs.) matrix, where each column has one (weighted) observation in the position corresponding to its sigma^2 bin   
databig=zeros(10,length(sigma2));
for i=1:length(sigma2)
    for j=1:length(sigbinrange(:,1))
        if sigma2(i)>=sigbinrange(j,1) && sigma2(i)<=sigbinrange(j,2)
            databig(j,i)=scalewts(i);
        end
    end
end

%% Generate gamma bins

% gdist are the breakpoints for the fine grid of risk aversion values
gdist=[(.5:.75:2),2.5,3,3.4,3.8,(4.5:1:7.5),8.5,9.5,10,10.5,11,12:1:16,18:1:34];

%% medsig2vals is just the median (sq. root) values of the 10 sigma2 bins

% change to weighted medsig2vals - calculated in s2tenbins.m
medsig2vals=medsig2vals_new;


%% gbins2 hold the breakpoints for the 4 bins (so 5 breakpoints)
gbins2=[0,2,3.84,7.52,10000];
 
%% Create "data" to be used in structural estimation. This is weighted # of observations in each of the 10x4 sigma2 x gamma cells. 
data1=gsPDF;

%% Here is measurement error code: Goal is to find f(log(1/gamma)|log(1/gammahat))
[fgonbin]=ln_gammainv_dist(mu,sig2x,sig2eps,gdist,gbins2);

%% Generate numerical risk penalty

% uncomment this when using static version of model:
logvzeromat=-.5.*(medsig2vals*gdist);

%% search for optimal beta/fsigzero

% makemodelparams solves for the betainv and fsgizero parameter values that maximize lnlhood function. It uses negtranslikelihood.m to do so.

options=optimset('MaxFunEvals',1e+13,'maxiter',250000,'TolFun',1e-8);

% Round 1
[mybetainv,myfsigzero,theta]=makemodelparams_structural(logvzeromat,.1,sum(data1,2).*exp(.5*5*medsig2vals.^2)/sum(sum(data1,2)),zeros(size(covmat(1,:))),...
    medsig2vals,gdist,fgonbin,riskavbin,databig,covmat,options);
% Round 2
[mybetainv,myfsigzero,theta]=makemodelparams_structural(logvzeromat,mybetainv,myfsigzero',...
    theta',medsig2vals,gdist,fgonbin,riskavbin,databig,covmat,options);
% Round 3
[mybetainv,myfsigzero,theta]=makemodelparams_structural(logvzeromat,mybetainv,myfsigzero',...
    theta',medsig2vals,gdist,fgonbin,riskavbin,databig,covmat,options);
% Round 4
[mybetainv,myfsigzero,theta]=makemodelparams_structural(logvzeromat,mybetainv,myfsigzero',...
    theta',medsig2vals,gdist,fgonbin,riskavbin,databig,covmat,options);
% Round 5
[mybetainv,myfsigzero,mytheta]=makemodelparams_structural(logvzeromat,mybetainv,myfsigzero',...
    theta',medsig2vals,gdist,fgonbin,riskavbin,databig,covmat,options);
myfsigzero=myfsigzero';



