function [f_ln_gammainv_bin]=ln_gammainv_dist(mu,sig2x,sig2eps,gdist,gbins2)

%% Goal of this code

% We want to find f(ln(1/gamma) | bin), from equation 35 in the paper. 

%% New code

% NOTE: lginv is the "flipped" vector -log(gdist). That is the lowest value of -log(gdist) is the largest value of gdist, 
% so the order of the vector is flipped so that the largest value of gdist is the first value input into -log(gdist). 
% This is so calculating probabilities of ln(1/gamma) via cdf differences makes sense. Also note: ln(1/gamma)=-ln(gamma).
% Most risk averse on the left, most risk tolerant on the right.
lginv=[-10,fliplr(-log(gdist)),10]';  % augmented with really large/small values for cdf calculations

% lgbreak are the breakpoints of the 4 gamma bins, transformed by taking ln(1/gamma)=-log(gamma).  
% The values min(lginv) and max(gbins2) approximate zero and infinity respectively.
% Most risk averse on the left, most risk tolerant on the right.
lgbreak=[min(lginv),fliplr(-log(gbins2(2:4))),10]';

% prob_bin_unc is the unconditional probability of \hat{ln(1/gamma)} landing in each of the 4 bins
prob_bin_unc=zeros(1,length(gbins2)-1);
for j=1:length(gbins2)-1
    prob_bin_unc(j)=normcdf(lgbreak(j+1),mu,sqrt(sig2x+sig2eps))-normcdf(lgbreak(j),mu,sqrt(sig2x+sig2eps));
end

% This section of the code is simply a check that f(ln(1/gamma)|bin) integrates to 1 for each bin, verifying it is the correct form for the pdf
X=(-10:.01:10)';
Y1=normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(2)-X,0,sqrt(sig2eps))-normcdf(lgbreak(1)-X,0,sqrt(sig2eps)))./(prob_bin_unc(1)));
Y2=normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(3)-X,0,sqrt(sig2eps))-normcdf(lgbreak(2)-X,0,sqrt(sig2eps)))./(prob_bin_unc(2)));
Y3=normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(4)-X,0,sqrt(sig2eps))-normcdf(lgbreak(3)-X,0,sqrt(sig2eps)))./(prob_bin_unc(3)));
Y4=normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(5)-X,0,sqrt(sig2eps))-normcdf(lgbreak(4)-X,0,sqrt(sig2eps)))./(prob_bin_unc(4)));
inttest1=trapz(X,Y1);  % should equal to 1
inttest2=trapz(X,Y2);  % should equal to 1
inttest3=trapz(X,Y3);  % should equal to 1
inttest4=trapz(X,Y4);  % should equal to 1

% This is f(ln(1/gamma)|bin) approximated by stipulating that Pr(ln(1/gamma_i)|bin)=int_{lginv(i)}^{lginv(i+1} f(ln(1/gamma_i)|bin).  
% Said differently, the probability of observing gamma_i is probability of observing anything between gamma_i and gamma_i+1
% (assuming zero at the bottom).  Column 1 corresponds to...

f_ln_gammainv_bin=zeros(length(gdist)+1,length(gbins2)-1);
for j=1:length(gbins2)-1
    for i=1:length(gdist)+1
        f_ln_gammainv_bin(i,j)=quad(@(x) normpdf(x,mu,sqrt(sig2x))...
            .*((normcdf(lgbreak(j+1)-x,0,sqrt(sig2eps))-normcdf(lgbreak(j)-x,0,sqrt(sig2eps)))./(prob_bin_unc(j))),lginv(i),lginv(i+1));
    end
    f_ln_gammainv_bin(:,j)=f_ln_gammainv_bin(:,j)./sum(f_ln_gammainv_bin(:,j));
end

%%%%%%%%% Now most risk averse is on right, least risk averse on left for 4 coarse bins
f_ln_gammainv_bin=fliplr(f_ln_gammainv_bin);
%%%%%%%%% Now low gammas on bottom, high gammas on top
f_ln_gammainv_bin=flipud(f_ln_gammainv_bin);

% f_ln_gammainv_bin should now line up with...


%% This part calculates E[gamma|(1/gamma)_bin]
X=(-10:.01:10)';

% risk aversion
z1=(1./exp(X)).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(2)-X,0,sqrt(sig2eps))-normcdf(lgbreak(1)-X,0,sqrt(sig2eps)))./(prob_bin_unc(1)));
z2=(1./exp(X)).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(3)-X,0,sqrt(sig2eps))-normcdf(lgbreak(2)-X,0,sqrt(sig2eps)))./(prob_bin_unc(2)));
z3=(1./exp(X)).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(4)-X,0,sqrt(sig2eps))-normcdf(lgbreak(3)-X,0,sqrt(sig2eps)))./(prob_bin_unc(3)));
z4=(1./exp(X)).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(5)-X,0,sqrt(sig2eps))-normcdf(lgbreak(4)-X,0,sqrt(sig2eps)))./(prob_bin_unc(4)));
E1=trapz(X,z1);
E2=trapz(X,z2);
E3=trapz(X,z3);
E4=trapz(X,z4);

% risk tolerance
t1=exp(X).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(2)-X,0,sqrt(sig2eps))-normcdf(lgbreak(1)-X,0,sqrt(sig2eps)))./(prob_bin_unc(1)));
t2=exp(X).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(3)-X,0,sqrt(sig2eps))-normcdf(lgbreak(2)-X,0,sqrt(sig2eps)))./(prob_bin_unc(2)));
t3=exp(X).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(4)-X,0,sqrt(sig2eps))-normcdf(lgbreak(3)-X,0,sqrt(sig2eps)))./(prob_bin_unc(3)));
t4=exp(X).*normpdf(X,mu,sqrt(sig2x)).*((normcdf(lgbreak(5)-X,0,sqrt(sig2eps))-normcdf(lgbreak(4)-X,0,sqrt(sig2eps)))./(prob_bin_unc(4)));
E1t=trapz(X,t1);
E2t=trapz(X,t2);
E3t=trapz(X,t3);
E4t=trapz(X,t4);



%save test_gamma_dist



%% Old code 
% %% Start calculations here
% 
% % NOTE: lginv is the "flipped" vector -log(gdist). That is the lowest value of -log(gdist) is the largest value of gdist, 
% % so the order of the vector is flipped so that the largest value of gdist is the first value of inputed into -log(gdist). 
% % This is so calculating probabilities of ln(1/gamma) via cdf differences makes sense. Also note: ln(1/gamma)=-ln(gamma).
% lginv=[-10000000,fliplr(-log(gdist)),100000000]';  % augmented with really large value for cdf calculations
% 
% % lgbreak is the set of gamma bin breakpoints, evaluated at -log() and flipped to match order of lginv
% lgbreak=[min(lginv),fliplr(-log(gbins2(2:4))),max(lginv)+1]';
% 
% % f_ln_one_over_gamma is F(ln(1/gamma_i))-F(ln(1/gamma_i-1)) where F is the normal cdf with mean mu and variance sigma2x. 
% % This will approximate f(ln(1/gamma)), the unconditional distribution of true log risk tolerance.  
% % There are 39 regions since there are 40 break points.
% f_ln_one_over_gamma=zeros(39,1);
% for i=2:40
%     f_ln_one_over_gamma(i-1)=normcdf(lginv(i),mu,sqrt(sig2x))-normcdf(lginv(i-1),mu,sqrt(sig2x));
% end
% 
% % prob_bin_given_gamma is probability of landing in a particular bin given gamma. It is numerator in ratio in eq. 35 of paper.  
% % It is normcdf(bin_up-ln(1/gamma))-normcdf(bin_low-ln(1/gamma)), where normal variable is error term e. 
% % Note as a check that all rows of febin sum to 1.
% prob_bin_given_gamma=zeros(39,4);
% for i=2:39
%     prob_bin_given_gamma(i-1,1)=normcdf(lgbreak(2)-lginv(i),0,sqrt(sig2eps))-normcdf(lgbreak(1)-lginv(i),0,sqrt(sig2eps));
%     prob_bin_given_gamma(i-1,2)=normcdf(lgbreak(3)-lginv(i),0,sqrt(sig2eps))-normcdf(lgbreak(2)-lginv(i),0,sqrt(sig2eps));
%     prob_bin_given_gamma(i-1,3)=normcdf(lgbreak(4)-lginv(i),0,sqrt(sig2eps))-normcdf(lgbreak(3)-lginv(i),0,sqrt(sig2eps));
%     prob_bin_given_gamma(i-1,4)=normcdf(lgbreak(5)-lginv(i),0,sqrt(sig2eps))-normcdf(lgbreak(4)-lginv(i),0,sqrt(sig2eps));
% end
% prob_bin_given_gamma(39,:)=[0,0,0,1];
% 
% % prob_bin_unc is the *unconditional* probability of landing in each of the 4 risk tolerance bins. 
% % Note as a check that prob_bin_unc sums to 1.  It is denominator in eq. 35 of paper
% prob_bin_unc=zeros(1,4);
% for j=1:4
%     prob_bin_unc(j)=normcdf(lgbreak(j+1),mu,sqrt(sig2x+sig2eps))-normcdf(lgbreak(j),mu,sqrt(sig2x+sig2eps));
% end
% 
% denom=sum(repmat(f_ln_one_over_gamma,1,4).*prob_bin_given_gamma,1);
% 
% % fgonbin is f(gamma|bin). Is 38x4?
% fgonbin=repmat(f_ln_one_over_gamma,1,4).*prob_bin_given_gamma./repmat(denom,39,1);




